import * as THREE from '../build/three.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';
import { MapControls } from './jsm/controls/OrbitControls.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        this.domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xcccccc);
        //scene.fog = new THREE.FogExp2(0xcccccc, 0.002);
        this._scene = scene;

        this._setupCamera();
        this._setupControls();
        this._setupModel();
        this._setupLight();
        this._setupGUI();
        
        window.onresize = this.resize.bind(this);

        this.resize();

        this.render();
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(60, window.innerWidth / window.innerHeight, 1, 10000);
        camera.position.set(400, 200, 0);
        this._camera = camera;
    }

    _setupControls() {
        const controls = new MapControls(this._camera, this._renderer.domElement);

        controls.enableDamping = true;
        controls.dampingFactor = 0.05;

        controls.screenSpacePanning = false;
        controls.minDistance = 100;
        controls.maxDistance = 1000;

        controls.maxPolarAngle = Math.PI / 2;

        this._controls = controls;
    }

    _setupModel() {
        const geometry = new THREE.BoxGeometry(1, 1, 1);
        geometry.translate(0, 0.5, 0);
        const material = new THREE.MeshPhongMaterial({ color: 0xffffff, flatShading: true });

        for(let i=0; i<500; i++) {
            const mesh = new THREE.Mesh(geometry, material);
            mesh.position.x = Math.random() * 1600 - 800;
            mesh.position.y = 0;
            mesh.position.z = Math.random() * 1600 - 800;
            mesh.scale.x = 20;
            mesh.scale.y = Math.random() * 80 + 10;
            mesh.scale.z = 20;
            mesh.updateMatrix(); // Updates the local transform.
            mesh.matrixAutoUpdate = false;
            this._scene.add(mesh);
        }
    }

    _setupLight() {
        const dirLight1 = new THREE.DirectionalLight(0xffffff);
        dirLight1.position.set(1, 1, 1);
        this._scene.add(dirLight1);

        const dirLight2 = new THREE.DirectionalLight(0x002288);
        dirLight2.position.set(-1, -1, -1);
        this._scene.add(dirLight2);

        const ambientLight = new THREE.AmbientLight(0x222222);
        this._scene.add(ambientLight);
    }

    update() {
        this._controls.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio(window.devicePixelRatio);
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }

    _setupGUI() {
        const gui = new GUI();
        gui.add(this._controls, "screenSpacePanning");
    }
}

window.onload = function () {
    (new App()).render();
}